// display color sequence of the PEACE flag on an RGB LED
// connect RGB LED directly to an Arduino UNO or ATINY13
#include <avr/pgmspace.h>

//pin connections of the RGB LED
#define MASK_R 0B00000001  // D8  on the uno, PB0 on the ATINY
#define MASK_G 0B00000010  // D9  on the uno, PB1 on the ATINY
#define MASK_B 0B00000100  // D10 on the uno, PB2 on the ATINY

//calibration of relative strength of red, green, blue
#define cal_R 1.0*255
#define cal_G 0.3*255
#define cal_B 0.5*255

//color definitions
#define ncol 10
#define color_BLACK   0
#define color_RED     1
#define color_GREEN   2
#define color_BLUE    3
#define color_YELLOW  4
#define color_CYAN    5
#define color_MAGENTA 6
#define color_WHITE   7
#define color_ORANGE  8
#define color_VIOLET  9

//definitions of the colors;
const byte rgb[ncol*3] PROGMEM = {
  0.0 * cal_R, 0.0 * cal_G, 0.0 * cal_B, //black
  1.0 * cal_R, 0.0 * cal_G, 0.0 * cal_B, //red
  0.0 * cal_R, 1.0 * cal_G, 0.0 * cal_B, //green
  0.0 * cal_R, 0.0 * cal_G, 1.0 * cal_B, //blue
  1.0 * cal_R, 1.0 * cal_G, 0.0 * cal_B, //yellow
  0.0 * cal_R, 1.0 * cal_G, 1.0 * cal_B, //cyan
  1.0 * cal_R, 0.0 * cal_G, 1.0 * cal_B, //magenta
  1.0 * cal_R, 1.0 * cal_G, 1.0 * cal_B, //white
  1.0 * cal_R, 0.5 * cal_G, 0.0 * cal_B, //orange
  0.5 * cal_R, 0.0 * cal_G, 1.0 * cal_B  //violet
};

//definition of the pattern to be displayed
#define nstep 10
const byte patt[nstep] PROGMEM = {
  color_VIOLET,
  color_BLUE,
  color_CYAN,
  color_GREEN,
  color_YELLOW,
  color_ORANGE,
  color_RED,
  color_BLACK,
  color_BLACK,
  color_BLACK,
};

// this parameter determines the speed
#define steptime 2400  //1s per step on 16MHz Arduino
//#define steptime 180     //1s per step on 1.2MHz ATTINY 

void setup() {
  //set LED pins to output and switch them off
  DDRB = (MASK_R | MASK_G | MASK_B);
  PORTB = 0;
}

byte istep = 0;

void loop() {

  //retrieve the color for this step
  byte col =pgm_read_byte_near(patt+istep);
  //find the corresponding intensities of red,green,blue 
  byte fr = pgm_read_byte_near(rgb+3*col+0);
  byte fg = pgm_read_byte_near(rgb+3*col+1);
  byte fb = pgm_read_byte_near(rgb+3*col+2);
  
  //light up the red, green and blue channel sequentially
  for (unsigned int i = 0; i < steptime; i++) {
    for (byte icnt = 0; icnt < 255; icnt++) {
      PORTB = MASK_R * (fr > icnt);
    }
    for (byte icnt = 0; icnt < 255; icnt++) {
      PORTB = MASK_G * (fg > icnt);
    }
    for (byte icnt = 0; icnt < 255; icnt++) {
      PORTB = MASK_B * (fb > icnt);
    }
  }

  //go to next step
  istep++;
  if (istep >= nstep) istep = 0;
}
